/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>
 * Ryan Barrett <p4@ryanb.org>
 * Maulik Shah <p4@maulik.net>
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file msg_data.h
 *
 * Defines the cmsg_data class, which is derived from the abstract cmsg class.
 * The cmsg_data class is the class that DATA messages instantiate. For more
 * information, see protocol.txt, msg.h, and protocol.h.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef _MSG_DATA_H
#define _MSG_DATA_H

#include "msg.h"
#include "protocol.h"
using namespace std;


/*****************************************************************************
 * FORWARD DECLARATIONS
 *****************************************************************************/
class cmsg_factory;  // so that we can declare it as a friend


/*****************************************************************************
 * CMSG_DATA CLASS
 *****************************************************************************/

class cmsg_data : public cmsg {
  public:

  /** ctor
   */
  cmsg_data(ushort plugin_id, const char *data, uint len, bool encrypted);

  /** dtor
   */
  ~cmsg_data() {}

  /** send, broadcast
   *
   * DATA messages can be broadcast or sent direct, so both the broadcast and
   * the send methods are made public.
   */
  virtual const cconn &send(const string &username) 
    { return cmsg::send(username); }
  virtual uint broadcast()
    { return cmsg::broadcast(); }

  protected:
  friend class cmsg_factory;  // for constructing from a p4 header

  /** ctors
   */
  cmsg_data(const p4_header &hdr) : cmsg(hdr) {}
  cmsg_data(const cmsg &msg) : cmsg(msg) {}

  /** virt_process
   *
   * For a DATA message, virt_process places the message on a receive queue.
   */
  virtual bool virt_process() const;
};


#endif  // _MSG_DATA_H
